import sexpdata
from sexpdata import Symbol
import ntpath
import os


def write_sym_lib_table(files):
	print("writing sym")
	s = [ Symbol('sym_lib_table'), [ Symbol('version'), 7 ] ]
	for path in files:
		fname = ntpath.basename(path)
		name = fname.replace(".kicad_sym", "")
		s.append([
			Symbol('lib'),
			[ Symbol('name'), name ],
			[ Symbol('type'), 'KiCad' ],
			[ Symbol('uri'), os.path.join('${KIPRJMOD}', path) ],
			[ Symbol('options'), '' ],
			[ Symbol('descr'), '' ],
		])
	with open('sym-lib-table', 'w') as f:
		sexpdata.dump(s, f)


def write_fp_lib_table(files):
	print("writing foot")
	s = [ Symbol('fp_lib_table'), [ Symbol('version'), 7 ] ]
	for path in files:
		fname = ntpath.basename(path)
		name = fname.replace(".pretty", "")
		s.append([
			Symbol('lib'),
			[ Symbol('name'), name ],
			[ Symbol('type'), 'KiCad' ],
			[ Symbol('uri'), os.path.join('${KIPRJMOD}', path) ],
			[ Symbol('options'), '' ],
			[ Symbol('descr'), '' ],
		])
	with open('fp-lib-table', 'w') as f:
		sexpdata.dump(s, f)




from . import registry


def run_command(args):
	packages = registry.listt()
	print_packages(packages)


def print_packages(packages):
	print(f"All packages in registry:")
	print()
	for name, package in packages.items():
		release = package['releases'][-1]
		print(f"{name}\tv{release['version']} - {release['author']} - {release['artifact_url']}")
	print()



import json
from . import config
import requests
from .registry import get_release_for
import os
import io
import zipfile
from . import kicad_project_tables
import glob
import shutil


def run_command(args):
	package_ref = args.package_ref

	if package_ref == '.':
		c = config.parse_config()
		deps = {}
		for depname, depversion in c.dependencies.items():
			explore_deps(depname, depversion, deps)
		install_deps(deps)


def explore_deps(name, version, found_packages={}):
	if name in found_packages:
		if found_packages[name] != version:
			raise Exception(f"incompatible version for {name}: {version}, committed version is {found_packages[name]['version']}")
		else:
			return found_packages

	release = get_release_for(name, version)
	found_packages[name] = release

	if 'dependencies' in release:
		for depname, depversion in release['dependencies'].items():
			explore_deps(depname, depversion, found_packages)

	return found_packages


def install_deps(deps):
	shutil.rmtree("./kpm_modules", ignore_errors=True)
	for package_name, release in deps.items():
		print(f"installing {package_name}")
		install_package(package_name, release['version'], release['artifact_url'])
	install_libraries()


def install_package(name, version, zip_url):
	package_dir = f"./kpm_modules/{name}@{version}/"
	os.makedirs(package_dir, exist_ok=True)
	res = requests.get(zip_url)
	r = zipfile.ZipFile(io.BytesIO(res.content)).extractall(package_dir)


def install_libraries():
	# link symbol files
	symfiles = glob.glob("kpm_modules/**/*.kicad_sym", recursive=True)
	kicad_project_tables.write_sym_lib_table(symfiles)

	# link footprint files
	footfiles = glob.glob("kpm_modules/**/*.pretty", recursive=True)
	kicad_project_tables.write_fp_lib_table(footfiles)

	# install 3d models
	# install spice models
	# install plugins




from . import registry


def run_command(args):
	package = registry.search(args.package_ref)
	print_package(args.package_ref, package)


def print_package(name, package):
	print(f"Package: {name}")
	print(f"Owner: {package['owner']}")
	print(f"Homepage: {package['homepage']}")
	print()
	print("Releases:")
	for release in package['releases']:
		print(f"v{release['version']} - {release['author']} - {release['artifact_url']}")
	print()



from functools import lru_cache
import pathlib
import os
import requests

temporary_registry = requests.get("https://raw.githubusercontent.com/danroblewis/kicad-package-index/main/registry.json").json()


@lru_cache
def get_release_for(package_name, version):
	if package_name not in temporary_registry:
		raise Exception(f"no package exists by name {package_name}")

	releases = temporary_registry[package_name]['releases']
	found = None
	for release in releases:
		if release['version'] == version:
			return release


def get_zip_url_for(package_name, version):
	release = get_release_for(package_name, version)
	return release['artifact_url']


def search(package_name):
	package = temporary_registry[package_name]
	return package


def listt():
	return temporary_registry




from . import config
import os

def run_command(args):
	# create empty json file

	c = config.Config()

	# create our kpm.json config file
	with open('kpm.json', 'w') as f:
		f.write(c.toJSON())

	# add to or create .gitignore
	if os.path.exists('.gitignore'):
		with open('.gitignore') as fr:
			if 'kpm_modules' not in fr.read():
				with open('.gitignore', 'a') as fw:
					fw.write("\nkpm_modules")
	else:
		with open('.gitignore', 'w') as f:
			f.write("kpm_modules\n")



